@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 #--------------------------------------------------------------------------------
-#www2sms.py v0.2.1
+#www2sms.py v1.0.1
 #Copyright Bjoern Olausson
 #--------------------------------------------------------------------------------
 #This program is free software; you can redistribute it and/or modify
@@ -24,10 +24,10 @@
 #
 
 import os, errno, pwd, sys, math, sqlite3, time, calendar, evolution, re
-from PyQt4 import QtCore, QtGui
-from www2sms_gui_main import *
-from www2sms_gui_widget_account import *
-from www2sms_gui_widget_advanced import *
+from PyQt4 import QtCore
+from PyQt4.QtGui import QApplication, QMainWindow, QDialog, QPushButton, QMessageBox
+from www2sms_gui_main import Ui_MainWindow
+from www2sms_gui_widget_account import Ui_FormAccount
 from www2sms_gui_dialog_progress import Ui_Form_Progress
 import www2sms_providers
 
@@ -44,24 +44,21 @@ for label in dir(www2sms_providers):
 	list_providers.append(provider.getName())
 	providers.append(provider)
 
-class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
+class MainWindow(QMainWindow, Ui_MainWindow):
 	def __init__(self, win_parent = None):
 		#Init the base class
-		QtGui.QMainWindow.__init__(self, win_parent)
+		QMainWindow.__init__(self, win_parent)
 		self.ui = Ui_MainWindow()
 		self.ui.setupUi(self)
 		# Setup GUI elements
+		# Get the phone book
+		self.my_phone_book = self.get_contacts()
 		# Populate the contact list
-		self.comboBox_contact_populate()
+		for contact in self.my_phone_book:
+			self.ui.comboBox_contact.addItem(contact[0])
 		# Populate the available Providers
 		for value in list_providers:
 			AccSet.ui.comboBox_SetupProvider.addItem(value)
-		CT = time.localtime(time.time())
-		# Set the year for scheduling SMS. From now to the next three years
-		YEAR = CT[0]
-		for YEARS in range(YEAR,YEAR + 3, 1):
-			AdvSet.ui.comboBox_AdvancedScheduleStartYear.addItem(str(YEARS))
-			AdvSet.ui.comboBox_AdvancedScheduleStopYear.addItem(str(YEARS))
 		# Try to update the GUI to the last used Account, else skip this
 		try:
 			PROVIDER, ALIAS, LOGIN, PASSWORD = c.execute('SELECT provider, alias, login, password FROM login WHERE last=1').fetchall()[0]
@@ -95,54 +92,54 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 		QtCore.QObject.connect(self.ui.plainTextEdit_SMS, QtCore.SIGNAL("textChanged()"), self.UpdateChars)
 		QtCore.QObject.connect(AccSet.ui.comboBox_SetupAlias, QtCore.SIGNAL("activated(QString)"), self.on_SetupAlias_activated)
 		QtCore.QObject.connect(AccSet.ui.pushButton_SetupDelete, QtCore.SIGNAL("clicked()"), self.on_SetupDelete_clicked)
-		QtCore.QObject.connect(AdvSet.ui.comboBox_AdvancedScheduleStartMonth, QtCore.SIGNAL("activated(QString)"), self.on_StartDate_changed)
-		QtCore.QObject.connect(AdvSet.ui.comboBox_AdvancedScheduleStopMonth, QtCore.SIGNAL("activated(QString)"), self.on_StopDate_changed)
-		QtCore.QObject.connect(AdvSet.ui.comboBox_AdvancedScheduleStartYear, QtCore.SIGNAL("activated(QString)"), self.on_StartDate_changed)
-		QtCore.QObject.connect(AdvSet.ui.comboBox_AdvancedScheduleStopYear, QtCore.SIGNAL("activated(QString)"), self.on_StopDate_changed)
 		self.enableTabs(provider_index)
+		self.cpsms(provider_index)
 		QtCore.QObject.connect(self.ui.actionAdv_Provider_Settings, QtCore.SIGNAL("triggered(bool)"), self.ShowDialogAdv)
 		QtCore.QObject.connect(self.ui.actionAccount_Setup, QtCore.SIGNAL("triggered(bool)"), self.ShowDialogAccSet)
+		self.cpsms(provider_index)
+    
 	def enableTabs(self, provider_index):
 		## Enable the SMS SEND Button if we have an account, otherwise not
-		if self.ACCOUNTS:
-			self.ui.pushButton_Send.show()
+		if AccSet.ui.comboBox_SetupAlias.currentText():
+			self.ui.pushButton_Send.setEnabled(True)
 		else:
-			self.ui.pushButton_Send.hide()
+		    	self.ui.pushButton_Send.setEnabled(False)
 		## Enable the "advanced" options if supported
 		if (provider_index >= 0) and providers[provider_index].hasAdvancedFeatures():
+			# TODO: replace "o2online_de" by the selected provider...
+			from www2sms_gui_widget_advanced_o2online_de import  Ui_FormAdvanced
+			class FormAdv(QDialog, Ui_FormAdvanced):
+				def __init__(self, win_parent = None):
+					#Init the base class
+					QDialog.__init__(self, win_parent)
+					self.ui = Ui_FormAdvanced()
+					self.ui.setupUi(self)
+			#The O2 Advanced Form
+			self.AdvSet = FormAdv()
+			providers[provider_index].SetupGUI(self)
 			self.ui.actionAdv_Provider_Settings.setEnabled(True)
 		else:
 			self.ui.actionAdv_Provider_Settings.setDisabled(True)
 
 	def ShowDialogAdv(self):
-	    	PROVIDER = AccSet.ui.comboBox_SetupProvider.currentText()
-	    	# TODO: When there are more providers, a GUI element should be created for every
-	    	# TODO: advanced setting. For now only o2online.de has such settings, so AdvSet
-	    	# TODO: can be considered the GUI for o2online advanced settings. Thats why we set
-	    	# TODO: PROVIDER = 'AdvSet'
-	    	PROVIDER = 'AdvSet'
-	    	# Dictionary of all available UIs for advanced settings.
-		p_dict = {'AdvSet' : AdvSet.show}
-		p_dict[PROVIDER]()
-		# TODO: Check wy the sheduling for o2online.de is not working.
-		# TODO: Meanwhile this is disabled.
-		AdvSet.ui.groupBox_AdvancedSchedule.setDisabled(True)
+		self.AdvSet.show()
 
 	def ShowDialogAccSet(self):
 		AccSet.show()
 
-	# Class wich is called to get the contacts from the evolution backend
-	def comboBox_contact_populate(self):
-	    	# TODO: Find out why Umlaute like "ä,ö,ü,ß" are garbled.
+	# Function wich generates a nice sorted list for all contacts.
+	def get_contacts(self):
 		# Regular expression to find all phone numbers in the vCard
 		re_tel = re.compile('TEL;TYPE')
 
+		# Not usefull right now...
 		name_attributes = [
 			"full-name",
 			"given-name",
 			"family-name",
 			"nickname"]
 
+		# Not usefull right now...
 		phone_attributes = [
 			"assistant-phone",
 			"business-phone",
@@ -164,27 +161,22 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 			"telex",
 			"tty",]
 
-		single_contact = []
-		single_contact_numbers = []
-		self.contact_list = []
-		self.phone_list = []
+		# Initiate some variables.
+		contact_and_numbers = []
+		# Used in another function
+		contacts_with_numbers = []
 
+		# Open the default address book
 		abook = evolution.ebook.open_addressbook("default")
 		contacts = abook.get_all_contacts()
+		# Pre sort the contacts.
 		contacts.sort(key=lambda obj: obj.get_property(name_attributes[2]))
 
+		# Here we check if the contact has Name, Family Name or alias set.
+		# If we have Name and Family Name, we store it in reverse order with ","
+		# Else we take what is available
 		for econtact in contacts:
-			single_contact = [econtact.get_property(name_attributes[2]), econtact.get_property(name_attributes[1]), econtact.get_property(name_attributes[3])]
-			vcard = econtact.get_vcard_string()
-			for line in vcard.split('\n'):
-				if re_tel.search(line):
-					single_contact_numbers.append(line.split(':')[1].strip())
-			self.contact_list.append(single_contact)
-			self.phone_list.append(single_contact_numbers)
-			single_contact = []
-			single_contact_numbers = []
-
-		for contact in self.contact_list:
+			contact = [econtact.get_property(name_attributes[2]), econtact.get_property(name_attributes[1]), econtact.get_property(name_attributes[3])]
 			if contact[0] and contact[1]:
 				item = "%s, %s" %(contact[0], contact[1])
 			elif contact[0] and not contact[1]:
@@ -195,59 +187,66 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 				item = "%s" %(contact[2])
 			else:
 				item = "No name available"
-			self.ui.comboBox_contact.addItem(item)
+			# Everything is stored in UTF-8 so we decode it for the QT-GUI
+			contact_and_numbers = [item.decode('utf-8')]
+
+			# Since python-evolution does not yield all contact numbers,
+			# we hack around this flaw with VCards.
+			vcard = econtact.get_vcard_string()
+			for line in vcard.split('\n'):
+				if re_tel.search(line):
+					number = line.split(':')[1].strip()
+					contact_and_numbers.extend([number.decode('utf-8')])
+			# Now store the list of contact with all its numbers in in another list
+			contacts_with_numbers.append(contact_and_numbers)
+
+		# Here we have to sort the list again, because of the contacts which only had an alias.
+		contacts_with_numbers = sorted(contacts_with_numbers)
+		# Return the final sorted list contacts_with_numbers:
+		# list = [ ['Family Name', 'Name', 'Number1', 'Number2', 'Number(n+1)'], [...], [...] ]
+		return contacts_with_numbers
 
 	def comboBox_contact_itemActivated(self):
 		self.ui.comboBox_number.clear()
 		contact_index = self.ui.comboBox_contact.currentIndex()
-		if len(self.phone_list[contact_index]) == 0:
-			self.ui.comboBox_number.addItem('No number available')
-		else:
-			for item in self.phone_list[contact_index]:
-				try:
-					if not item:
-						self.ui.comboBox_number.addItem('No number available')
-					else:
-						self.ui.comboBox_number.addItem(item)
-				except:
-					pass
-		if len(self.phone_list[contact_index]) == 0:
+		for item in self.my_phone_book[contact_index][1:]:
+			try:
+				if not item:
+					self.ui.comboBox_number.addItem('No number available')
+				else:
+					self.ui.comboBox_number.addItem(item)
+			except:
+				pass
+		if len(self.my_phone_book[contact_index][1:]) == 0:
 			self.ui.lineEdit_Number.setText('No number available')
 		else:
-			self.ui.lineEdit_Number.setText(self.phone_list[contact_index][0])
+			self.ui.lineEdit_Number.setText(self.my_phone_book[contact_index][1])
 
 	def comboBox_number_itemActivated(self):
 		item = self.ui.comboBox_number.currentText()
 		self.ui.lineEdit_Number.setText(item)
 
-	# Method which is called to get the proper number of days for the schedule start date
-	def on_StartDate_changed(self):
-		cal = calendar.Calendar()
-		sYEAR = int(self.ui.comboBox_AdvancedScheduleStartYear.currentText())
-		sMONTH = int(self.ui.comboBox_AdvancedScheduleStartMonth.currentText())
-		self.ui.comboBox_AdvancedScheduleStartDay.clear()
-		for DAY in cal.itermonthdays(sYEAR, sMONTH):
-			if DAY != 00:
-				DAY = '%02d' %(DAY)
-				self.ui.comboBox_AdvancedScheduleStartDay.addItem(str(DAY))
+	# Method which is called to get the number of chars per SMS for the current provider.
+	# This method must be called whenever the provider in the setup was changed or altered.
+	def cpsms(self, provider_index):
+	    	self.cps, self.max_c= providers[provider_index].chars_per_sms()
+	    	self.cps += 1
 
-	# Method which is called to get the proper number of days for the schedule stop date
-	def on_StopDate_changed(self):
-		cal = calendar.Calendar()
-		sYEAR = int(self.ui.comboBox_AdvancedScheduleStopYear.currentText())
-		sMONTH = int(self.ui.comboBox_AdvancedScheduleStopMonth.currentText())
-		self.ui.comboBox_AdvancedScheduleStopDay.clear()
-		for DAY in cal.itermonthdays(sYEAR, sMONTH):
-			if DAY != 00:
-				DAY = '%02d' %(DAY)
-				self.ui.comboBox_AdvancedScheduleStopDay.addItem(str(DAY))
-
-	# Method which is called to get the number of characters typed in plainTextEdit_SMS and update lcdNumber_SMSChars
+	# Method which is called to get the number of characters typed in plainTextEdit_SMS,
+	# update lcdNumber_SMSChars and check if chars/sms hav been exceeded
 	def UpdateChars(self):
 		SMS_CHARS = self.ui.plainTextEdit_SMS.document().characterCount() - 1
 		self.ui.label_SMSChars.setText(str(SMS_CHARS))
+		CharsLeft = self.max_c - SMS_CHARS
+		self.ui.label_SMSCharsLeft.setText(str(CharsLeft))
+		if SMS_CHARS > self.max_c and self.ui.pushButton_Send.isEnabled():
+		    	QMessageBox.information(self, 'www2sms', "Reached maximum allowd characters (%s)" %(self.max_c))
+		    	self.ui.pushButton_Send.setEnabled(False)
+		# Here we have to check if an Account exist, if not, we will not enable the send Button
+		if not self.ui.pushButton_Send.isEnabled() and SMS_CHARS <= self.max_c and AccSet.ui.comboBox_SetupAlias.currentText():
+			self.ui.pushButton_Send.setEnabled(True)
 		# Might be better. Floor to Ceiling division: -(-x//y)
-		SMS_COUNT = int(math.ceil(SMS_CHARS / 160) + 1)
+		SMS_COUNT = int(math.ceil(SMS_CHARS / self.cps) + 1)
 		self.ui.label_SMSCount.setText(str(SMS_COUNT))
 
 	# Method which updates the GUI to the Account selected in comboBox_SetupAlias
@@ -266,6 +265,7 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 		AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
 		AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
 		self.enableTabs(provider_index)
+		self.cpsms(provider_index)
 		# Remove the last account flag to set it on the new/updated profile
 		c.execute('UPDATE login SET last=0 WHERE last=1')
 		# Set the last flag on the new Account
@@ -292,7 +292,8 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 			self.enableTabs(-1)
 		else:
 			AccSet.on_SetupAlias_activated()
-		QtGui.QMessageBox.information(AccSet, 'www2sms', "Account %s deleted" %(ALIAS))
+		
+		QMessageBox.information(AccSet, 'www2sms', "Account %s deleted" %(ALIAS))
 
 	# Method which is called, when an account is created/updated
 	def on_SetupSave_clicked(self):
@@ -313,7 +314,7 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 		    	AccSet.ui.comboBox_SetupAlias.addItem(ALIAS)
 		conn.commit()
 		# Let the user know that we have saved the the account/changes
-		QtGui.QMessageBox.information(AccSet, 'www2sms', "saved")
+		QMessageBox.information(AccSet, 'www2sms', "saved")
 		# Update the GUI
 		alias_index = AccSet.ui.comboBox_SetupAlias.findText(ALIAS)
 		AccSet.ui.comboBox_SetupAlias.setCurrentIndex(alias_index)
@@ -322,91 +323,84 @@ class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
 		AccSet.ui.lineEdit_SetupLogin.setText(LOGIN)
 		AccSet.ui.lineEdit_SetupPassword.setText(PASSWORD)
 		self.enableTabs(provider_index)
+		self.cpsms(provider_index)
 
 	# Method which is called when we want to send the sms. It actually calls a method on the providers class
 	def on_send_clicked(self):
-		self.ui.pushButton_Send.setText("Sending...")
-		# Really ready to send?
-		reply = QtGui.QMessageBox.question(self, 'Send SMS?', "Are you sure?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
-		# If reply is YES, we send the SMS, else we just do nothing
-		if reply == QtGui.QMessageBox.Yes:
-			self.Stat = FormPro()
-		    # Read Scheduling settings if activated
-			SCHEDULE = []
-			if AdvSet.ui.checkBox_AdvancedScheduleRepeat.isChecked():
-				# SCHEDULE Layout: ([RECURRENCY, STOPTrueFalse], [YEARstart, YEARstop], [MONTHstart, MONTHstop], [DAYstart, DAYstop], [HOUREstart, HOUREstop], [MINUTEstart, MINUTEstop])
-				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedRecurrence.currentText()), self.ui.checkBox_AdvancedScheduleStop.isChecked() ])
-				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedScheduleStartYear.currentText()), str(AdvSet.ui.comboBox_AdvancedScheduleStopYear.currentText()) ])
-				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedScheduleStartMonth.currentText()), str(AdvSet.ui.comboBox_AdvancedScheduleStopMonth.currentText()) ])
-				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedScheduleStartDay.currentText()), str(AdvSet.ui.comboBox_AdvancedScheduleStopDay.currentText()) ])
-				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedScheduleStartHoure.currentText()), str(AdvSet.ui.comboBox_AdvancedScheduleStopHoure.currentText()) ])
-				SCHEDULE.append([ str(AdvSet.ui.comboBox_AdvancedScheduleStartMinute.currentText()), str(AdvSet.ui.comboBox_AdvancedScheduleStopMinute.currentText()) ])
+	    	# Read the provider to use
+		PROVIDER = unicode(AccSet.ui.comboBox_SetupProvider.currentText())
+		provider_index = AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
+		provider = providers[provider_index]
+	    	#Check if a receiver number was provided
+		# Read the number to which we send the SMS
+		NUMBER = unicode(self.ui.lineEdit_Number.text())
+		# Check if an number was entered at all
+	    	if not NUMBER:
+		    	QMessageBox.warning(self, 'www2sms', "Please enter an number")
+			return
+		# Run the provider specific number checks 
+		stat, FIXED_NUMBER, message = providers[provider_index].num_regexp(NUMBER)
+		if not stat:
+			QMessageBox.warning(self, 'www2sms', message)
+			return
+		if FIXED_NUMBER != NUMBER:
+		    	reply = QMessageBox.question(self, 'www2sms', "Correcte number?\n%s --> %s" %(NUMBER, FIXED_NUMBER), QMessageBox.Yes, QMessageBox.No)
+		    	if reply == QMessageBox.Yes:
+			    	NUMBER = FIXED_NUMBER
+				self.ui.lineEdit_Number.setText(NUMBER)
 			else:
-				SCHEDULE = False
-
-			# Read Sender
-			if AdvSet.ui.radioButton_AdvancedMyMobileNumber.isChecked():
-				SENDER = "MyNumber"
-			elif AdvSet.ui.radioButton_AdvancedAnonym.isChecked():
-				SENDER = "Anonym"
-			elif AdvSet.ui.radioButton_AdvancedText.isChecked():
-				SENDER = self.ui.lineEdit_AdvancedText.text()
-
-			# Read Special
-			FLASH = AdvSet.ui.checkBox_AdvancedFlashSMS.isChecked()
+				return
+		# Check if there is some text to send at all
+		# Read the text we want to send
+		SMSTEXT = unicode(self.ui.plainTextEdit_SMS.toPlainText())
+		if not SMSTEXT:
+			QMessageBox.information(self, 'www2sms', "Please enter some text")
+			return
+		self.ui.pushButton_Send.setText("Sending...")
+		self.ui.pushButton_Send.setEnabled(False)
+		# Really ready to send?
+		reply = QMessageBox.question(self, 'Send SMS?', "Are you sure?", QMessageBox.Yes, QMessageBox.No)
+		# If reply is YES, we send the SMS, else we just do nothing
+		if reply == QMessageBox.Yes:
+			self.Stat = FormPro()
 			# Read the login
 			LOGIN = unicode(AccSet.ui.lineEdit_SetupLogin.text())
 			# Read the password
 			PASSWORD = unicode(AccSet.ui.lineEdit_SetupPassword.text())
-			# Read the number to which we send the SMS
-			NUMBER = unicode(self.ui.lineEdit_Number.text())
-			# Read the text we want to send
-			SMSTEXT = unicode(self.ui.plainTextEdit_SMS.toPlainText())
-			# Read the provider to use
-			PROVIDER = unicode(AccSet.ui.comboBox_SetupProvider.currentText())
-			# Get the provder
-			provider_index = AccSet.ui.comboBox_SetupProvider.findText(PROVIDER)
-			provider = providers[provider_index]
 			# send the SMS
-			STATUS, MSG = provider.sendSMS(self, app, NUMBER, SMSTEXT, LOGIN, PASSWORD, SCHEDULE, SENDER, FLASH)
+			STATUS, MSG = provider.sendSMS(self, app, NUMBER, SMSTEXT, LOGIN, PASSWORD)
 			self.Stat.close()
-			if STATUS:
+			if STATUS == True:
 				self.on_send_success(MSG)
 			else:
 				self.on_send_failed(MSG)
+		self.ui.pushButton_Send.setEnabled(True)
 		self.ui.pushButton_Send.setText("Send")
 
 	# Method which is called from providers class when the sms was successfully send
 	def on_send_success(self,MSG):
-		QtGui.QMessageBox.information(self, 'www2sms', MSG)
+		QMessageBox.information(self, 'www2sms', MSG)
 		self.ui.lineEdit_Number.clear()
 		self.ui.plainTextEdit_SMS.clear()
 
 	# Method which is called from providers class when the sms failed to send
 	def on_send_failed(self,MSG):
-		QtGui.QMessageBox.warning(self, 'www2sms', MSG)
+		QMessageBox.warning(self, 'www2sms', MSG)
 
 def DialogStatusMessage(self, statusmessage):
-	QtGui.QMessageBox.information(self, 'www2sms', statusmessage)
+	QMessageBox.information(self, 'www2sms', statusmessage)
 
-class FormAccount(QtGui.QDialog, Ui_FormAccount):
+class FormAccount(QDialog, Ui_FormAccount):
 	def __init__(self, win_parent = None):
 		#Init the base class
-		QtGui.QDialog.__init__(self, win_parent)
+		QDialog.__init__(self, win_parent)
 		self.ui = Ui_FormAccount()
 		self.ui.setupUi(self)
 
-class FormAdv(QtGui.QDialog, Ui_FormAdvanced):
+class FormPro(QDialog, Ui_Form_Progress):
 	def __init__(self, win_parent = None):
 		#Init the base class
-		QtGui.QDialog.__init__(self, win_parent)
-		self.ui = Ui_FormAdvanced()
-		self.ui.setupUi(self)
-
-class FormPro(QtGui.QDialog, Ui_Form_Progress):
-	def __init__(self, win_parent = None):
-		#Init the base class
-		QtGui.QDialog.__init__(self, win_parent)
+		QDialog.__init__(self, win_parent)
 		self.ui = Ui_Form_Progress()
 		self.ui.setupUi(self)
 
@@ -441,13 +435,11 @@ c = conn.cursor()
 if __name__ == "__main__":
 	# Someone is launching this directly
 	# Create the QApplication
-	app = QtGui.QApplication(sys.argv)
+	app = QApplication(sys.argv)
 	#The Status Form
 	#Stat = FormPro()
 	#The Account Form
 	AccSet = FormAccount()
-	#The O2 Advanced Form
-	AdvSet = FormAdv()
 	#The Main window
 	ui = MainWindow()
 	ui.show()
